@@ -10,12 +10,24 @@ import sys
 from abc import ABC
 from datetime import datetime
 from enum import Enum
-from typing import ClassVar, Dict, List, Optional, Tuple, Type, Union, get_type_hints
+from typing import (
+    ClassVar,
+    Dict,
+    List,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    Union,
+    get_type_hints,
+)
 
+import backoff
 import gcsfs
 import humanize
 import pendulum
 from google.cloud import storage
+from google.cloud.storage import Blob
 from pydantic import (
     BaseModel,
     Extra,
@@ -160,6 +172,21 @@ class GTFSFeedType(str, Enum):
         raise RuntimeError(f"managed to end up with an invalid enum type of {self}")
 
 
+def set_metadata(blob: storage.Blob, model: BaseModel, exclude=None):
+    blob.metadata = {PARTITIONED_ARTIFACT_METADATA_KEY: model.json(exclude=exclude)}
+    blob.patch()
+
+
+# Is there a better pattern for making this retry optional by the caller?
+@backoff.on_exception(
+    backoff.expo,
+    exception=(Exception,),
+    max_tries=2,
+)
+def set_metadata_with_retry(*args, **kwargs):
+    return set_metadata(*args, **kwargs)
+
+
 class PartitionedGCSArtifact(BaseModel, abc.ABC):
     """
     This class is designed to be subclassed to model "extracts", i.e. a particular
@@ -228,7 +255,14 @@ class PartitionedGCSArtifact(BaseModel, abc.ABC):
     def path(self):
         return os.path.join(self.bucket, self.name)
 
-    def save_content(self, content: bytes, exclude=None, fs: gcsfs.GCSFileSystem = None, client: storage.Client = None):
+    def save_content(
+        self,
+        content: bytes,
+        exclude=None,
+        fs: gcsfs.GCSFileSystem = None,
+        client: storage.Client = None,
+        retry_metadata: bool = False,
+    ):
         if (fs is None) == (client is None):
             raise TypeError("must provide a gcsfs file system OR a storage client")
 
@@ -254,20 +288,22 @@ class PartitionedGCSArtifact(BaseModel, abc.ABC):
                 client=client,
             )
 
-            blob.metadata = {PARTITIONED_ARTIFACT_METADATA_KEY: self.json(exclude=exclude)}
-            blob.patch()
+            set_metadata_func = set_metadata_with_retry if retry_metadata else set_metadata
+            set_metadata_func(
+                blob=blob,
+                model=self,
+                exclude=exclude,
+            )
 
 
 # TODO: this should really use a typevar
 def fetch_all_in_partition(
     cls: Type[PartitionedGCSArtifact],
-    fs: gcsfs.GCSFileSystem,
     partitions: Dict[str, PartitionType],
     bucket: str = None,
     table: str = None,
     verbose=False,
-    progress=False,
-) -> List[PartitionedGCSArtifact]:
+) -> Tuple[List[PartitionedGCSArtifact], List[Blob], List[Blob]]:
     if not bucket:
         bucket = cls.bucket
 
@@ -295,16 +331,20 @@ def fetch_all_in_partition(
     files = client.list_blobs(re.sub(r"^gs://", "", bucket), prefix=prefix, delimiter=None)
 
     parsed: List[PartitionedGCSArtifact] = []
+    blobs_with_missing_metadata: List[Blob] = []
+    blobs_with_invalid_metadata: List[Blob] = []
 
     for file in files:
         try:
             parsed.append(parse_obj_as(cls, json.loads(file.metadata[PARTITIONED_ARTIFACT_METADATA_KEY])))
-        except (TypeError, KeyError) as e:
-            raise RuntimeError(f"metadata missing on {bucket}/{file.name}") from e
-        except ValidationError as e:
-            raise RuntimeError(f"invalid metadata found on {bucket}/{file.name}") from e
+        except (TypeError, KeyError):
+            logging.exception(f"metadata missing on {bucket}/{file.name}")
+            blobs_with_missing_metadata.append(file)
+        except ValidationError:
+            logging.exception(f"invalid metadata found on {bucket}/{file.name}")
+            blobs_with_invalid_metadata.append(file)
 
-    return parsed
+    return parsed, blobs_with_missing_metadata, blobs_with_invalid_metadata
 
 
 class ProcessingOutcome(BaseModel, abc.ABC):
@@ -506,6 +546,7 @@ class GTFSDownloadConfig(BaseModel, extra=Extra.forbid):
     schedule_url_for_validation: Optional[HttpUrl]
     auth_query_params: Dict[str, str] = {}
     auth_headers: Dict[str, str] = {}
+    computed: bool = False
 
     @validator("extracted_at", allow_reuse=True)
     def coerce_extracted_at(cls, v):
@@ -529,7 +570,7 @@ class GTFSDownloadConfig(BaseModel, extra=Extra.forbid):
 
         return v
 
-    def build_request(self, auth_dict: dict) -> Request:
+    def build_request(self, auth_dict: Mapping[str, str]) -> Request:
         params = {k: auth_dict[v] for k, v in self.auth_query_params.items()}
         headers = {k: auth_dict[v] for k, v in self.auth_headers.items()}
 
@@ -608,6 +649,7 @@ class GTFSScheduleFeedExtract(GTFSFeedExtract):
     table: ClassVar[str] = GTFSFeedType.schedule
     feed_type: ClassVar[str] = GTFSFeedType.schedule
     partition_names: ClassVar[List[str]] = ["dt", "ts", "base64_url"]
+    reconstructed: bool = False
 
     @validator("config", allow_reuse=True)
     def is_schedule_type(cls, v: GTFSDownloadConfig):
@@ -641,7 +683,7 @@ class GTFSRTFeedExtract(GTFSFeedExtract):
 
 def download_feed(
     config: GTFSDownloadConfig,
-    auth_dict: Dict,
+    auth_dict: Mapping[str, str],
     ts: pendulum.DateTime,
     default_filename="feed",
 ) -> Tuple[GTFSFeedExtract, bytes]:
@@ -695,7 +737,7 @@ if __name__ == "__main__":
     # whereas UTC serializes to 2022-08-18T00:00:00Z
     # this should probably be checked in the partitioned artifact?
     yesterday_noon = pendulum.yesterday("Etc/UTC").replace(minute=0, second=0, microsecond=0)
-    vp_files = fetch_all_in_partition(
+    vp_files, _, _ = fetch_all_in_partition(
         cls=GTFSRTFeedExtract,
         fs=get_fs(),
         partitions={
@@ -706,7 +748,7 @@ if __name__ == "__main__":
         verbose=True,
         progress=True,
     )
-    schedule_files = fetch_all_in_partition(
+    schedule_files, _, _ = fetch_all_in_partition(
         cls=GTFSScheduleFeedExtract,
         fs=get_fs(),
         partitions={
